@@ -9,6 +9,7 @@ no if $] >= 5.018, warnings => "experimental::smartmatch";
 use Carp qw(cluck);
 use Encode qw(decode encode);
 use Travel::Routing::DE::EFA::Route;
+use Travel::Routing::DE::EFA::Route::Message;
 use LWP::UserAgent;
 use XML::LibXML;
 
@@ -40,7 +41,7 @@ use Exception::Class (
 	},
 );
 
-our $VERSION = '2.11';
+our $VERSION = '2.12';
 
 sub set_time {
 	my ( $self, %conf ) = @_;
@@ -497,6 +498,41 @@ sub itdtime_str {
 		$node->getAttribute('minute') );
 }
 
+sub parse_cur_info {
+	my ( $self, $node ) = @_;
+
+	my $xp_text     = XML::LibXML::XPathExpression->new('./infoLinkText');
+	my $xp_subject  = XML::LibXML::XPathExpression->new('./infoText/subject');
+	my $xp_subtitle = XML::LibXML::XPathExpression->new('./infoText/subtitle');
+	my $xp_content  = XML::LibXML::XPathExpression->new('./infoText/content');
+
+	my $e_text     = ( $node->findnodes($xp_text) )[0];
+	my $e_subject  = ( $node->findnodes($xp_subject) )[0];
+	my $e_subtitle = ( $node->findnodes($xp_subtitle) )[0];
+	my $e_content  = ( $node->findnodes($xp_content) )[0];
+
+	my %msg = (
+		summary     => $e_text->textContent,
+		subject     => $e_subject->textContent,
+		subtitle    => $e_subtitle->textContent,
+		raw_content => $e_content->textContent,
+	);
+	for my $key ( keys %msg ) {
+		$msg{$key} = decode( 'UTF-8', $msg{$key} );
+	}
+	return Travel::Routing::DE::EFA::Route::Message->new(%msg);
+}
+
+sub parse_reg_info {
+	my ( $self, $node ) = @_;
+
+	my %msg = (
+		summary => decode( 'UTF-8', $node->textContent ),
+	);
+
+	return Travel::Routing::DE::EFA::Route::Message->new(%msg);
+}
+
 sub parse_xml_part {
 	my ( $self, $route ) = @_;
 
@@ -514,10 +550,15 @@ sub parse_xml_part {
 	  = XML::LibXML::XPathExpression->new('./itdDateTimeTarget/itdDate');
 	my $xp_stime
 	  = XML::LibXML::XPathExpression->new('./itdDateTimeTarget/itdTime');
-	my $xp_mot   = XML::LibXML::XPathExpression->new('./itdMeansOfTransport');
+	my $xp_mot = XML::LibXML::XPathExpression->new('./itdMeansOfTransport');
+	my $xp_fp  = XML::LibXML::XPathExpression->new('./itdFootPathInfo');
+	my $xp_fp_e
+	  = XML::LibXML::XPathExpression->new('./itdFootPathInfo/itdFootPathElem');
 	my $xp_delay = XML::LibXML::XPathExpression->new('./itdRBLControlled');
-	my $xp_info
+
+	my $xp_sched_info
 	  = XML::LibXML::XPathExpression->new('./itdInfoTextList/infoTextListElem');
+	my $xp_cur_info = XML::LibXML::XPathExpression->new('./infoLink');
 
 	my $xp_mapitem_rm = XML::LibXML::XPathExpression->new(
 		'./itdMapItemList/itdMapItem[@type="RM"]/itdImage');
@@ -557,11 +598,14 @@ sub parse_xml_part {
 		my $e_astime  = ( $e_arr->findnodes($xp_stime) )[0];
 		my $e_mot     = ( $e->findnodes($xp_mot) )[0];
 		my $e_delay   = ( $e->findnodes($xp_delay) )[0];
-		my @e_info    = $e->findnodes($xp_info);
+		my $e_fp      = ( $e->findnodes($xp_fp) )[0];
+		my @e_sinfo   = $e->findnodes($xp_sched_info);
+		my @e_cinfo   = $e->findnodes($xp_cur_info);
 		my @e_dmap_rm = $e_dep->findnodes($xp_mapitem_rm);
 		my @e_dmap_sm = $e_dep->findnodes($xp_mapitem_sm);
 		my @e_amap_rm = $e_arr->findnodes($xp_mapitem_rm);
 		my @e_amap_sm = $e_arr->findnodes($xp_mapitem_sm);
+		my @e_fp_e    = $e->findnodes($xp_fp_e);
 
 		# not all EFA services distinguish between scheduled and realtime
 		# data. Set sdate / stime to date / time when not provided.
@@ -608,6 +652,22 @@ sub parse_xml_part {
 			$hash->{$key} = decode( 'UTF-8', $hash->{$key} );
 		}
 
+		if ($e_fp) {
+
+			# Note that position=IDEST footpaths are coupled with a special
+			# "walking" connection, so their duration is already known and
+			# accounted for. However, we still save it here, since
+			# detecting and handling this is the API client's job (for now).
+			$hash->{footpath_type}     = $e_fp->getAttribute('position');
+			$hash->{footpath_duration} = $e_fp->getAttribute('duration');
+			for my $e (@e_fp_e) {
+				push(
+					@{ $hash->{footpath_parts} },
+					[ $e->getAttribute('type'), $e->getAttribute('level') ]
+				);
+			}
+		}
+
 		$hash->{departure_routemaps}   = \@dep_rms;
 		$hash->{departure_stationmaps} = \@dep_sms;
 		$hash->{arrival_routemaps}     = \@arr_rms;
@@ -641,7 +701,9 @@ sub parse_xml_part {
 			);
 		}
 
-		$hash->{extra} = [ map { decode( 'UTF-8', $_->textContent ) } @e_info ];
+		$hash->{regular_notes}
+		  = [ map { $self->parse_reg_info($_) } @e_sinfo ];
+		$hash->{current_notes} = [ map { $self->parse_cur_info($_) } @e_cinfo ];
 
 		push( @route_parts, $hash );
 	}
@@ -855,7 +917,7 @@ Travel::Routing::DE::EFA - unofficial interface to EFA-based itinerary services
 
 =head1 VERSION
 
-version 2.11
+version 2.12
 
 =head1 DESCRIPTION
 
@@ -991,7 +1053,9 @@ Set walk speed.  Default: B<normal>
 
 =item B<with_bike> => B<0>|B<1>
 
-If true: Prefer connections allowing passengers with bikes
+If true: Request connections allowing passengers with bikes. Note that the
+backed may return an empty result if no such connection exists or bike-support
+simply isn't known.
 
 =item B<lwp_options> => I<\%hashref>
 
