@@ -6,14 +6,16 @@ use 5.010;
 
 use parent 'Class::Accessor';
 
-our $VERSION = '2.11';
+our $VERSION = '2.12';
 
 Travel::Routing::DE::EFA::Route::Part->mk_ro_accessors(
 	qw(arrival_platform arrival_stop
 	  arrival_date arrival_time arrival_sdate arrival_stime delay
 	  departure_platform
 	  departure_stop departure_date departure_time departure_sdate
-	  departure_stime train_destination train_line train_product
+	  departure_stime
+	  footpath_duration footpath_type
+	  train_destination train_line train_product
 	  )
 );
 
@@ -71,10 +73,40 @@ sub departure_stop_and_platform {
 	return $self->departure_stop;
 }
 
+sub footpath_parts {
+	my ($self) = @_;
+
+	if ( $self->{footpath_parts} ) {
+		return @{ $self->{footpath_parts} };
+	}
+	return;
+}
+
+# DEPRECATED
 sub extra {
 	my ($self) = @_;
 
-	return @{ $self->{extra} // [] };
+	my @ret = map { $_->summary } @{ $self->{regular_notes} // [] };
+
+	return @ret;
+}
+
+sub regular_notes {
+	my ($self) = @_;
+
+	if ( $self->{regular_notes} ) {
+		return @{ $self->{regular_notes} };
+	}
+	return;
+}
+
+sub current_notes {
+	my ($self) = @_;
+
+	if ( $self->{current_notes} ) {
+		return @{ $self->{current_notes} };
+	}
+	return;
 }
 
 sub via {
@@ -96,8 +128,11 @@ points, without interchanges
 
 	for my $part ( $route->parts ) {
 
-		if ( $part->extra ) {
-			say join( "\n", $part->extra );
+		if ( $part->regular_notes ) {
+			say join( "\n", $part->regular_notes );
+		}
+		if ( $part->current_notes ) {
+			say join( "\n", map [ $_->{summary} ] $part->current_notes );
 		}
 
 		printf(
@@ -110,7 +145,7 @@ points, without interchanges
 
 =head1 VERSION
 
-version 2.11
+version 2.12
 
 =head1 DESCRIPTION
 
@@ -169,6 +204,14 @@ station, usually outlining how to transfer from this train to the next one
 List of URLs, may be empty. Each URL points to an HTML map of the arrival
 station.
 
+=item $part->current_notes
+
+Remarks about unscheduled changes to the line serving this connaction part,
+such as cancelled stops. Most times, the EFA service does not include this
+information in its route calculations.
+
+Returns a list of Travel::Routing::DE::EFA::Route::Message(3pm) objects.
+
 =item $part->delay
 
 delay in minutes, 0 if unknown
@@ -212,10 +255,42 @@ departure station, usually outlining how to transfer from thep previous train
 List of URLs, may be empty. Each URL poinst to an HTML map of the departure
 station.
 
-=item $part->extra
+=item $part->footpath_duration
 
-Additional information about the connection.  Returns a list of
-newline-terminated strings
+Walking duration when transferring before / after / during this trip
+in minutes. The meaning depends on the value of B<footpath_type>.
+
+=item $part->footpath_parts
+
+Returns a list of [I<type>, I<level>] arrayrefs describing the
+footpath. For instance, ["ESCALATOR", "UP"], ["LEVEL", "LEVEL"],
+["STAIRS", "UP"] means first taking an escalator up, then walking a while,
+and then taking a flight of stairs up again.
+
+The content of I<type> and I<level> comes directly from the EFA backend. At
+the moment, the following values are known:
+
+=over
+
+=item type: ESCALATOR, LEVEL, STAIRS
+
+=item level: DOWN, LEVEL, UP
+
+=back
+
+=item $part->footpath_type
+
+type of this footpath, passed through from the EFA backend. The value
+"AFTER" indicates a footpath (transfer) after this route part. The value
+"IDEST" indicates that this route part already is a footpath (aka a walking
+connection between two stops), so the B<footpath> accessors contain redundant
+information. Other values such as "BEFORE" may also be returned, but this is
+unknown at this point.
+
+=item $part->regular_notes
+
+Remarks about the line serving this connaction part. Returns a list of
+Travel::Routing::DE::EFA::Route::Message(3pm) objects.
 
 =item $part->train_destination
 
@@ -260,7 +335,8 @@ $part->via does not work reliably.
 
 =head1 SEE ALSO
 
-Travel::Routing::DE::EFA(3pm), Class::Accessor(3pm).
+Travel::Routing::DE::EFA::Route::Message(3pm), Travel::Routing::DE::EFA(3pm),
+Class::Accessor(3pm).
 
 =head1 AUTHOR
 
